package fr.inria.phoenix.diasuite.framework.context.event;

import java.io.Serializable;

/**
 * An object to store a value published by the context <code>Event</code>.
 *
 * <pre>
 * context Event as String indexed by periodicity as Integer {
 * 	when provided message from Input
 * 	maybe publish;	
 * }
 * </pre>
 */
public final class EventValue implements Serializable {
    private static final long serialVersionUID = 0;
    
    private java.lang.String value;
    
    /**
     * Get the value of the context <code>Event</code>
     * 
     * @return the value of the context <code>Event</code>
     */
    public java.lang.String value() {
        return value;
    }
    
    private EventIndices indices;
    
    /**
     * Get the value of the indices of the context <code>Event</code>
     * 
     * @return the value of the indices
     */
    public EventIndices indices() {
        return indices;
    }
    
    public EventValue(java.lang.String value, EventIndices indices) {
        this.value = value;
        this.indices = indices;
    }
    
    public EventValue(java.lang.String value,
            java.lang.Integer periodicity) {
        this.value = value;
        this.indices = new EventIndices(periodicity);
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        result = prime * result + ((indices == null) ? 0 : indices.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        EventValue other = (EventValue) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        if (indices == null) {
            if (other.indices != null)
                return false;
        } else if (!indices.equals(other.indices))
            return false;
        return true;
    }
}
