package fr.inria.phoenix.diasuite.framework.context.position;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.pallettracker.PositionFromPalletTracker;

/**
 * * PalletTracker contexts
 * The provided position is per station and the computed one is per device.
 * 
 * <pre>
 * context Position as GPSCoordinates indexed by id as String {
 * 	when provided position from PalletTracker
 * 	no publish;
 * 	when required;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractPosition extends Service {
    
    public AbstractPosition(ServiceConfiguration serviceConfiguration) {
        super("/Context/Position/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverPalletTrackerForSubscribe.all().subscribePosition(); // subscribe to position from all PalletTracker devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("position") && source.isCompatible("/Device/Device/PhysicalDevice/Sensor/PalletTracker/")) {
            PositionFromPalletTracker positionFromPalletTracker = new PositionFromPalletTracker(this, source, (fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates) value,
                    (java.util.List<java.lang.String>) indexes[0]);
            
            onPositionFromPalletTracker(positionFromPalletTracker);
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("position")) {
            return getValue((java.lang.String) indexes[0]);
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private java.util.Map<PositionIndices, fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates> contextValues = new java.util.HashMap<PositionIndices, fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates>();
    
    private void setPosition(fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates newContextValue, java.lang.String id) {
        PositionIndices indices = new PositionIndices(id);
        contextValues.put(indices, newContextValue);
        getProcessor().publishValue(getOutProperties(), "position", newContextValue, id);
    }
    
    /**
     * Get the last value of the context
     * 
     * @param id the index <code>id</code> for the last value
     * @return the latest value published by the context for the given indexes
     */
    protected final fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates getLastValue(java.lang.String id) {
        PositionIndices indices = new PositionIndices(id);
        return contextValues.get(indices);
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>PalletTracker</code> device on which we have subscribed publish on its <code>position</code> source.
     * 
     * <pre>
     * when provided position from PalletTracker
     * 	no publish;
     * </pre>
     * 
     * @param positionFromPalletTracker the value of the <code>position</code> source and the <code>PalletTracker</code> device that published the value.
     */
    protected abstract void onPositionFromPalletTracker(PositionFromPalletTracker positionFromPalletTracker);
    
    /**
     * This method is called when the value of this context is required.
     * 
     * <pre>
     * when required;
     * </pre>
     * 
     * @param id the value of the <code>id</code> index
     * @return the value of the context to return
     */
    protected abstract fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates getValue(java.lang.String id);
    
    // End of interaction contract implementation
    
    // Discover part for PalletTracker devices
    /**
     * Use this object to discover PalletTracker devices.
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     * 
     * @see PalletTrackerDiscoverer
     */
    protected final PalletTrackerDiscoverer discoverPalletTrackerForSubscribe = new PalletTrackerDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>PalletTracker</code> devices that can be discovered
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerDiscoverer {
        private Service serviceParent;
        
        private PalletTrackerDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private PalletTrackerComposite instantiateComposite() {
            return new PalletTrackerComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices
         * 
         * @return a {@link PalletTrackerComposite} object composed of all discoverable <code>PalletTracker</code>
         */
        public PalletTrackerComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>PalletTracker</code> devices
         * 
         * @return a {@link PalletTrackerProxy} object pointing to a random discoverable <code>PalletTracker</code> device
         */
        public PalletTrackerProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>PalletTracker</code> devices
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerComposite extends fr.inria.diagen.core.service.composite.Composite<PalletTrackerProxy> {
        private PalletTrackerComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/PalletTracker/");
        }
        
        @Override
        protected PalletTrackerProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new PalletTrackerProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>id</code>.
         */
        public PalletTrackerComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>location</code>.
         */
        public PalletTrackerComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>user</code>.
         */
        public PalletTrackerComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>position</code> source. After a call to this method, the context will be notified when a
         * <code>PalletTracker</code> device of this composite publishes a value on its <code>position</code> source.
         */
        public void subscribePosition() {
            subscribeValue("position");
        }
        
        /**
         * Unsubscribes from the <code>position</code> source. After a call to this method, the context will no more be notified
         * when a <code>PalletTracker</code> device of this composite publishes a value on its <code>position</code> source.
         */
        public void unsubscribePosition() {
            unsubscribeValue("position");
        }
    }
    
    /**
     * A proxy to one <code>PalletTracker</code> device
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerProxy extends Proxy {
        private PalletTrackerProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>position</code> source. After a call to this method, the context will be notified when the
         * <code>PalletTracker</code> device of this proxy publishes a value on its <code>position</code> source.
         */
        public void subscribePosition() {
            subscribeValue("position");
        }
        
        /**
         * Unsubscribes from the <code>position</code> source. After a call to this method, the context will no more be notified
         * when the <code>PalletTracker</code> device of this proxy publishes a value on its <code>position</code> source.
         */
        public void unsubscribePosition() {
            unsubscribeValue("position");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for PalletTracker devices
}
