package fr.inria.phoenix.diasuite.framework.context.position;

import java.io.Serializable;

/**
 * An object to store a value published by the context <code>Position</code>.
 * <p>
 * * PalletTracker contexts
 * The provided position is per station and the computed one is per device.
 *
 * <pre>
 * context Position as GPSCoordinates indexed by id as String {
 * 	when provided position from PalletTracker
 * 	no publish;
 * 	when required;
 * }
 * </pre>
 */
public final class PositionValue implements Serializable {
    private static final long serialVersionUID = 0;
    
    private fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value;
    
    /**
     * Get the value of the context <code>Position</code>
     * 
     * <p>
     * * PalletTracker contexts
     * The provided position is per station and the computed one is per device.
     * 
     * @return the value of the context <code>Position</code>
     */
    public fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value() {
        return value;
    }
    
    private PositionIndices indices;
    
    /**
     * Get the value of the indices of the context <code>Position</code>
     * 
     * @return the value of the indices
     */
    public PositionIndices indices() {
        return indices;
    }
    
    public PositionValue(fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value, PositionIndices indices) {
        this.value = value;
        this.indices = indices;
    }
    
    public PositionValue(fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value,
            java.lang.String id) {
        this.value = value;
        this.indices = new PositionIndices(id);
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        result = prime * result + ((indices == null) ? 0 : indices.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        PositionValue other = (PositionValue) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        if (indices == null) {
            if (other.indices != null)
                return false;
        } else if (!indices.equals(other.indices))
            return false;
        return true;
    }
}
