package fr.inria.phoenix.diasuite.framework.context.state;

import java.io.Serializable;

/**
 * An object to store a value published by the context <code>State</code>.
 * <p>
 * context Warehouse as String indexed by position as GPSCoordinates {
 * when required get warehouse from PalletTrackingDB;
 * }
 *
 * <pre>
 * context State as PalletState {
 * 	when provided state from PalletTracker 
 * 	get Position, Temperature	 
 * 	always publish;
 * }
 * </pre>
 */
public final class StateValue implements Serializable {
    private static final long serialVersionUID = 0;
    
    private fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState value;
    
    /**
     * Get the value of the context <code>State</code>
     * 
     * <p>
     * context Warehouse as String indexed by position as GPSCoordinates {
     * when required get warehouse from PalletTrackingDB;
     * }
     * 
     * @return the value of the context <code>State</code>
     */
    public fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState value() {
        return value;
    }
    
    public StateValue(fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState value) {
        this.value = value;
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        StateValue other = (StateValue) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        return true;
    }
}
