package fr.inria.phoenix.diasuite.framework.controller.insert;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.state.StateValue;

/**
 * * PalletTracker controllers
 * 
 * <pre>
 * controller Insert {
 * 	when provided State
 * 	do InsertPalletState on PalletTrackingDB;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractInsert extends Service {
    
    public AbstractInsert(ServiceConfiguration serviceConfiguration) {
        super("/Controller/Insert/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("state", "/Context/State/"); // subscribe to State context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("state") && source.isCompatible("/Context/State/")) {
            StateValue stateValue = new StateValue((fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState) value);
            
            onState(stateValue, new DiscoverForState());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>State</code> context publishes a value.
     * 
     * <pre>
     * when provided State
     * 	do InsertPalletState on PalletTrackingDB;
     * </pre>
     * 
     * @param state the value of the <code>State</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onState(StateValue state, DiscoverForState discover);
    
    // End of interaction contract implementation
    
    // Discover object for State
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided State
     * 	do InsertPalletState on PalletTrackingDB;
     * </code>
     */
    protected final class DiscoverForState {
        private final PalletTrackingDBDiscovererForState palletTrackingDBDiscoverer = new PalletTrackingDBDiscovererForState(AbstractInsert.this);
        
        /**
         * @return a {@link PalletTrackingDBDiscovererForState} object to discover <code>PalletTrackingDB</code> devices
         */
        public PalletTrackingDBDiscovererForState palletTrackingDBs() {
            return palletTrackingDBDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>PalletTrackingDB</code> devices to execute action on for the
     * <code>when provided State</code> interaction contract.
     * <p>
     * ------------------
     * DB
     * ------------------------------------
     * 
     * <pre>
     * device PalletTrackingDB extends Service {
     * 	source warehouse as String indexed by position as GPSCoordinates;
     * 	action InsertPalletState;
     * }
     * </pre>
     */
    protected final static class PalletTrackingDBDiscovererForState {
        private Service serviceParent;
        
        private PalletTrackingDBDiscovererForState(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private PalletTrackingDBCompositeForState instantiateComposite() {
            return new PalletTrackingDBCompositeForState(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTrackingDB</code> devices
         * 
         * @return a {@link PalletTrackingDBCompositeForState} object composed of all discoverable <code>PalletTrackingDB</code>
         */
        public PalletTrackingDBCompositeForState all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>PalletTrackingDB</code> devices
         * 
         * @return a {@link PalletTrackingDBProxyForState} object pointing to a random discoverable <code>PalletTrackingDB</code> device
         */
        public PalletTrackingDBProxyForState anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTrackingDB</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link PalletTrackingDBCompositeForState} object composed of all matching <code>PalletTrackingDB</code> devices
         */
        public PalletTrackingDBCompositeForState whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>PalletTrackingDB</code> devices to execute action on for the
     * <code>when provided State</code> interaction contract.
     * <p>
     * ------------------
     * DB
     * ------------------------------------
     * 
     * <pre>
     * device PalletTrackingDB extends Service {
     * 	source warehouse as String indexed by position as GPSCoordinates;
     * 	action InsertPalletState;
     * }
     * </pre>
     */
    protected final static class PalletTrackingDBCompositeForState extends fr.inria.diagen.core.service.composite.Composite<PalletTrackingDBProxyForState> {
        private PalletTrackingDBCompositeForState(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/PalletTrackingDB/");
        }
        
        @Override
        protected PalletTrackingDBProxyForState instantiateProxy(RemoteServiceInfo rsi) {
            return new PalletTrackingDBProxyForState(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link PalletTrackingDBCompositeForState}, filtered using the attribute <code>id</code>.
         */
        public PalletTrackingDBCompositeForState andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Executes the <code>insert(value as PalletState)</code> action's method on all devices of this composite.
         * 
         * @param value the <code>value</code> parameter of the <code>insert(value as PalletState)</code> method.
         */
        public void insert(fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState value) throws InvocationException {
            launchDiscovering();
            for (PalletTrackingDBProxyForState proxy : proxies) {
                proxy.insert(value);
            }
        }
    }
    
    /**
     * A proxy to one <code>PalletTrackingDB</code> device to execute action on for the
     * <code>when provided State</code> interaction contract.
     * <p>
     * ------------------
     * DB
     * ------------------------------------
     * 
     * <pre>
     * device PalletTrackingDB extends Service {
     * 	source warehouse as String indexed by position as GPSCoordinates;
     * 	action InsertPalletState;
     * }
     * </pre>
     */
    protected final static class PalletTrackingDBProxyForState extends Proxy {
        private PalletTrackingDBProxyForState(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>insert(value as PalletState)</code> action's method on this device.
         * 
         * @param value the <code>value</code> parameter of the <code>insert(value as PalletState)</code> method.
         */
        public void insert(fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState value) throws InvocationException {
            callOrder("insert", value);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover object for State
}
