package fr.inria.phoenix.diasuite.framework.device.pallettracker;

import java.io.Serializable;

import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

/**
 * An object to store a value published on the source <code>position</code> of the device <code>PalletTracker</code>.
 *
 * <pre>
 * device PalletTracker extends Sensor {
 * 	source state as State;
 * 	source position as GPSCoordinates indexed by params as String [];
 * }
 * </pre>
 */
public final class PositionFromPalletTracker implements Serializable {
    private static final long serialVersionUID = 0;
    
    private fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value;
    
    /**
     * Get the value of the source <code>position</code>
     * 
     * @return the value of the source <code>position</code>
     */
    public fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value() {
        return value;
    }
    
    private PositionIndices indices;
    
    /**
     * Get the value of the indices of the source <code>position</code>
     * 
     * @return the value of the indices
     */
    public PositionIndices indices() {
        return indices;
    }
    
    public PositionFromPalletTracker(Service service, RemoteServiceInfo remoteServiceInfo, fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value, PositionIndices indices) {
        this.sender = new PalletTrackerProxy(service, remoteServiceInfo);
        this.value = value;
        this.indices = indices;
    }
    
    public PositionFromPalletTracker(Service service, RemoteServiceInfo remoteServiceInfo, fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates value,
            java.util.List<java.lang.String> params) {
        this.sender = new PalletTrackerProxy(service, remoteServiceInfo);
        this.value = value;
        this.indices = new PositionIndices(params);
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        result = prime * result + ((indices == null) ? 0 : indices.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        PositionFromPalletTracker other = (PositionFromPalletTracker) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        if (indices == null) {
            if (other.indices != null)
                return false;
        } else if (!indices.equals(other.indices))
            return false;
        return true;
    }
    
    // Proxy to the sender, i.e., the device that published the source
    private PalletTrackerProxy sender;
    
    /**
     * Get the sender of the source <code>position</code>. I.e., the <code>PalletTracker</code> device that published the source.
     * 
     * @return A proxy to the <code>PalletTracker</code> that triggered the source
     */
    public PalletTrackerProxy sender() {
        return sender;
    }
    
    /**
     * A proxy to a <code>PalletTracker</code> that discloses subscription/unsubscription methods.
     */
    public class PalletTrackerProxy extends Proxy {
        private PalletTrackerProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subcribes to publication of source <code>position</code> from this <code>PalletTracker</code>.
         */
        public void subscribePosition() {
            getService().getProcessor().subscribeValue(getService().getOutProperties(), new fr.inria.diagen.core.service.filter.SubscriptionFilter(this.getRemoteServiceInfo()), "position");
        }
        
        /**
         * Unsubcribes from publication of source <code>position</code> from this <code>PalletTracker</code>.
         */
        public void unsubscribePosition() {
            getService().getProcessor().unsubscribeValue(getService().getOutProperties(), new fr.inria.diagen.core.service.filter.SubscriptionFilter(this.getRemoteServiceInfo()), "position");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
}
