package fr.inria.phoenix.scenario.pallettracker.impl.context;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.phoenix.diasuite.framework.context.alert.AbstractAlert;
import fr.inria.phoenix.diasuite.framework.datatype.state.State;
import fr.inria.phoenix.diasuite.framework.device.pallettracker.StateFromPalletTracker;
import fr.inria.phoenix.diasuite.framework.device.timer.TimerTriggeredFromTimer;

public class Alert extends AbstractAlert {

	private Map<String, State> deviceStateMap;

	public Alert(ServiceConfiguration serviceConfiguration) {
		super(serviceConfiguration);

		deviceStateMap = Collections
				.synchronizedMap(new HashMap<String, State>());
	}

	@Override
	protected AlertValuePublishable onTimerTriggeredFromTimer(
			TimerTriggeredFromTimer timerTriggeredFromTimer) {

		if (timerTriggeredFromTimer.value().equals("lowbattery")) {
			StringBuffer lowBatteryDevices = new StringBuffer();
			lowBatteryDevices
					.append("The following devices are running on low battery : ");

			synchronized (deviceStateMap) {
				for (Entry<String, State> deviceStateEntry : deviceStateMap
						.entrySet())
					if (deviceStateEntry.getValue().getBatteryLevel()
							.equals("low"))
						lowBatteryDevices.append(deviceStateEntry.getKey()
								+ " | ");

				return new AlertValuePublishable(lowBatteryDevices.toString(),
						true);
			}
		}

		return new AlertValuePublishable(new String(), false);
	}

	@Override
	protected void onStateFromPalletTracker(
			StateFromPalletTracker stateFromPalletTracker) {

		String deviceId = stateFromPalletTracker.sender().id();
		State state = stateFromPalletTracker.value();

		synchronized (deviceStateMap) {
			deviceStateMap.put(deviceId, state);
		}
	}
}
