import static fr.inria.phoenix.diasuite.framework.mocks.Mock.*;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import fr.inria.phoenix.diasuite.framework.datatype.contact.Contact;
import fr.inria.phoenix.diasuite.framework.datatype.file.File;
import fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates;
import fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState;
import fr.inria.phoenix.diasuite.framework.datatype.state.State;
import fr.inria.phoenix.diasuite.framework.datatype.temperature.Temperature;
import fr.inria.phoenix.diasuite.framework.datatype.temperatureunit.TemperatureUnit;
import fr.inria.phoenix.diasuite.framework.mocks.InputMock;
import fr.inria.phoenix.diasuite.framework.mocks.MessengerMock;
import fr.inria.phoenix.diasuite.framework.mocks.PalletTrackerMock;
import fr.inria.phoenix.diasuite.framework.mocks.PalletTrackingDBMock;
import fr.inria.phoenix.diasuite.framework.mocks.TemperatureSensorMock;
import fr.inria.phoenix.diasuite.framework.mocks.TimerMock;
import fr.inria.phoenix.scenario.pallettracker.impl.ComponentBinder;

public class PalletTracker {

	private PalletTrackerMock pt1;
	private TemperatureSensorMock ts1;
	private PalletTrackingDBMock ptDB;
	private InputMock input;
	private MessengerMock messenger;
	private TimerMock timer;

	private Float tx1 = 28.189260f;
	private Float tx2 = 28.195777f;
	private Float tx3 = 28.195771f;
	private Float tx4 = 28.189013f;
	private Float tx5 = 28.189013f;
	private Float tx6 = 28.189013f;
	private Float tx7 = 28.18255f;

	private Float ty1 = -25.868423f;
	private Float ty2 = -25.861644f;
	private Float ty3 = -25.866365f;
	private Float ty4 = -25.858386f;
	private Float ty5 = -25.858386f;
	private Float ty6 = -25.858386f;
	private Float ty7 = -25.87912f;

	private Float s1 = -82f;
	private Float s2 = -85f;
	private Float s3 = -90f;
	private Float s4 = -91f;
	private Float s5 = -93f;
	private Float s6 = -94f;
	private Float s7 = -99f;

	@Before
	public void setUp() throws Exception {
		TIMEOUT = 1000;
		underTest(ComponentBinder.class);
	}

	@After
	public void tearDown() throws Exception {
		shutdown();
	}

	@Test
	public void triangulationComputation() {
		pt1 = mockPalletTracker("123", "here", "user");
		ts1 = mockTemperatureSensor("123", "here", "user");
		ptDB = mockPalletTrackingDB("1234");

		pt1.position(new GPSCoordinates(ty1, tx1), new ArrayList<String>() {
			{
				add("1431119746");
				add(s1.toString());
				add("0​5​B​1​");
			}
		});

		pt1.position(new GPSCoordinates(ty2, tx2), new ArrayList<String>() {
			{
				add("1431119747");
				add(s2.toString());
				add("0​5​B​2​");
			}
		});

		pt1.position(new GPSCoordinates(ty3, tx3), new ArrayList<String>() {
			{
				add("1431119748");
				add(s3.toString());
				add("0​5​B​3");
			}
		});

		pt1.position(new GPSCoordinates(ty4, tx4), new ArrayList<String>() {
			{
				add("1431119749");
				add(s4.toString());
				add("0​5​B4​");
			}
		});

		pt1.position(new GPSCoordinates(ty5, tx5), new ArrayList<String>() {
			{
				add("1431119750");
				add(s5.toString());
				add("0​5​B5​");
			}
		});

		pt1.position(new GPSCoordinates(ty6, tx6), new ArrayList<String>() {
			{
				add("1431119751");
				add(s6.toString());
				add("0​5​B46");
			}
		});

		pt1.position(new GPSCoordinates(ty7, tx7), new ArrayList<String>() {
			{
				add("1431119752");
				add(s7.toString());
				add("0​5​B7");
			}
		});

		ts1.temperature(new Temperature(25.0f, TemperatureUnit.CELSIUS));
		pt1.state(new State("OK", "1431119755", "ok"));

		assertTrue(ptDB.expectInsert(new PalletState(new GPSCoordinates(
				new Float(-25.9), new Float(28.2)), 25, "1431119755")));
	}

	@Test
	public void alertTest() {
		input = mockInput("123");
		pt1 = mockPalletTracker("542", "here", "default");
		ptDB = mockPalletTrackingDB("1234");
		timer = mockTimer("658");
		messenger = mockMessenger("201", "here", "default");

		pt1.state(new State("OK", "1431119755", "low"));
		input.message("event:lowbattery;1", new ArrayList<String>());

		timer.expectPeriodicSchedule();
		messenger.expectSendMessage(new Contact(), "Alert",
				"The following devices are running on low battery : 542 | ",
				new ArrayList<File>());
	}
}
